'use strict';

const Generator = require('yeoman-generator');
const mkdirp = require('mkdirp');
const commandExists = require('command-exists').sync;
const config = require('./config');
const { isNil } = require('lodash');

module.exports = class extends Generator {
  constructor() {
    super(...arguments);
    for(let optionName in config.options) {
      this.option(optionName, config.options[optionName]);
    }
  }

  intializing() {
    this.pkg = require('../../package.json');
  }

  async prompting() {
    this.answers = await this.prompt(config.prompts);
  }

  writing() {
    const templateData = {
      projectName: this._projectName(),
      version: this.pkg.version
    };

    const copy = (input, output) => {
      this.fs.copy(this.templatePath(input), this.destinationPath(output));
    }

    const copyTpl = (input, output, data) => {
      this.fs.copyTpl(
        this.templatePath(input),
        this.destinationPath(output),
        data
      );
    };

    config.filesToCopy.forEach(file => {
      copy(file.input, file.output);
    });

    config.filesToRender.forEach(file => {
      copyTpl(file.input, file.output, templateData);
    });

    config.dirsToCreate.forEach(item => {
      mkdirp(item);
    });

    if(this.options['with-mocks']) {
      copy('tests/helpers/mocks/index.ts', 'tests/helpers/mocks/index.ts');
    }
  }

  install() {
    const hasYarn = commandExists('yarn');
    this.installDependencies({
      npm: !hasYarn,
      yarn: hasYarn,
      bower: false
    });

    const gitRemote = this.options['gitRemote'];

    if(!commandExists('git') || isNil(gitRemote)) {
      return;
    }

    this.spawnCommandSync('git', ['init']);
    this.spawnCommandSync('git', ['remote', 'add', 'origin', gitRemote]);
  }

  _projectName() {
    return `@ormus/${this.answers['projectName']}`;
  }
}